from .modules import StochasticResnetModule
from .Resnet import Resnet


class StochasticResnet(Resnet):
    def __init__(self, num_classes: int):
        super().__init__()
        self.model = StochasticResnetModule(num_classes)



from typing import List
from collections import OrderedDict
from copy import deepcopy
from ...models.wrappers import ModelWrapper


class RecordsStateDictionaries:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.state_dicts: List[OrderedDict] = []

    def record(self, wrapper: ModelWrapper):
        if self.model is None:
            self.model = deepcopy(wrapper.model)
        self.state_dicts.append(wrapper.model.state_dict())

    def push(self, *args, **kwargs):
        self.record(*args, **kwargs)

    def add(self, *args, **kwargs):
        self.record(*args, **kwargs)



import torch
from .concerns import (
    ReportsSize,
    RecordsStateDictionaries,
    ReportsClassificationMetrics,
)
from ..models import Model


class Ensemble(ReportsSize, ReportsClassificationMetrics, RecordsStateDictionaries):
    def __init__(self, variant: Model):
        super().__init__()
        self.model = variant().model

    def logits(self, x: torch.Tensor) -> torch.Tensor:
        batch_size = x.shape[0]
        logits = torch.zeros((batch_size, self.num_classes))
        for model in self.models:
            logits += model(x)
        return logits

    def predict(self, x: torch.Tensor) -> torch.Tensor:
        logits = self.logits(x)
        return torch.nn.functional.softmax(logits, dim=1)
